"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateBuildResult = exports.generateDevBuildResult = exports.generateProdBuildResultWithEncrypt = exports.generateProdBuildResult = exports.generateProdUrl = void 0;
const ramda_1 = __importDefault(require("ramda"));
const path_1 = __importDefault(require("path"));
const encrypt_1 = require("./encrypt");
const urlQuery_1 = require("./urlQuery");
const logger_1 = require("./logger");
function generateProdUrl({ extType, extUuid, registerComponent, SDKVersion, md5, rnbaseurl, rnbasemd5, RNVersion, persistRnModuleName, }) {
    let values = {
        rnext: 1,
        hyaction: "newrn",
        // 1. 不要轻易改变这个 rnmodule 格式, 可能会影响到后端的统计
        // 2. 这个名称不要包含非 ascii 值
        rnmodule: persistRnModuleName || `${extUuid}-${extType}-${extUuid}`,
        rnentry: registerComponent,
        sdkversion: SDKVersion,
        rnmd5: md5,
        framework: RNVersion || process.env.HYEXT_RN_VER || "0",
    };
    if (rnbaseurl && rnbasemd5) {
        values = {
            ...values,
            rnbaseurl,
            rnbasemd5,
        };
    }
    return Object.entries(values)
        .map(([k, v]) => `${k}=${encodeURIComponent(v)}`)
        .join("&");
}
exports.generateProdUrl = generateProdUrl;
const generateDevUrl = (component, platform) => {
    return `rnentry=${component}&platform=${platform}&rnext=1&hyaction=newrn`;
};
async function generateProdBuildResult({ builderName, extInfo, buildResultConfig, onUrlCreateBefore = opts => opts, extendPlatformConfig = async (cfg) => cfg, extendExtTypeConfig = async (cfg) => cfg, }) {
    const config = buildResultConfig.map((cfg) => ({
        ...cfg,
        registerComponents: cfg.registerComponents.map(({ name, queryString }) => ({
            name,
            queryString: queryString ||
                generateProdUrl(onUrlCreateBefore({
                    ...extInfo,
                    extType: cfg.extType,
                    registerComponent: name,
                    SDKVersion: cfg.SDKVersion,
                    md5: cfg.bundleMd5,
                    rnbaseurl: cfg.baseUrl,
                    rnbasemd5: cfg.baseMd5,
                }, cfg.platform)),
        })),
    }));
    return await generateBuildResult({
        builderName,
        buildResultConfig: config,
        extendPlatformConfig,
        extendExtTypeConfig,
    });
}
exports.generateProdBuildResult = generateProdBuildResult;
async function generateProdBuildResultWithEncrypt({ builderName, extInfo, buildResultConfig, onUrlCreateBefore = (opts) => opts, extendExtTypeConfig, extendPlatformConfig, // 为 webview proxy 等操作开个口子
outputBasePath }) {
    return generateProdBuildResult({
        builderName,
        extInfo,
        buildResultConfig,
        onUrlCreateBefore,
        extendPlatformConfig: async (cfg, platform, extType) => {
            const inputFile = path_1.default.join(outputBasePath, cfg.path);
            // 尽量文件名少出现rn的关键字眼
            const outPath = path_1.default.join(outputBasePath, "horizon", extType.replace(/rn/ig, 'fk'));
            const res = await (0, encrypt_1.encryptBundle)(inputFile, outPath, platform);
            // if the result is null, skip it
            if (res == null)
                return cfg;
            if (Array.isArray(cfg.entries) && cfg.entries.length > 0) {
                const copy = { ...cfg.entries[0] };
                // 修改 url 上的 md5
                const query = (0, urlQuery_1.toQueryObj)(copy.url);
                query.rnmd5 = res.md5;
                copy.url = (0, urlQuery_1.toQueryStr)(query, false);
                cfg.encryption = [
                    {
                        encryptType: "zip",
                        encryptContent: {
                            path: path_1.default.relative(outputBasePath, res.outputFilePath),
                            entries: [copy],
                        },
                    },
                ];
            }
            else {
                (0, logger_1.dbg)("PlatformConfig数据异常");
            }
            return extendPlatformConfig ? extendPlatformConfig(cfg, platform, extType) : cfg;
        },
        extendExtTypeConfig,
    });
}
exports.generateProdBuildResultWithEncrypt = generateProdBuildResultWithEncrypt;
async function generateDevBuildResult({ builderName, buildResultConfig, extendExtTypeConfig, extendPlatformConfig, }) {
    const config = buildResultConfig.map((cfg) => ({
        ...cfg,
        registerComponents: cfg.registerComponents.map(({ name, queryString }) => ({
            name,
            queryString: queryString || generateDevUrl(name, cfg.platform),
        })),
    }));
    return await generateBuildResult({
        builderName,
        buildResultConfig: config,
        extendPlatformConfig,
        extendExtTypeConfig,
    });
}
exports.generateDevBuildResult = generateDevBuildResult;
async function generateBuildResult({ builderName, buildResultConfig, extendPlatformConfig = async (cfg) => cfg, extendExtTypeConfig = async (cfg) => cfg, }) {
    async function getBuildResult(info) {
        const { extType, bundleType = "RN_BUNDLE", SDKVersion, publicPath, platform, bundleRelativePath, registerComponents } = info;
        return {
            [extType]: await extendExtTypeConfig({
                type: bundleType,
                builder: builderName,
                content: {
                    sdkVersion: SDKVersion,
                    baseURI: publicPath || "",
                    // @ts-ignore
                    rnBundle: {
                        [platform]: await extendPlatformConfig({
                            path: bundleRelativePath,
                            entries: registerComponents.map(({ name, queryString }) => ({
                                name,
                                url: queryString,
                            })),
                        }, platform, extType),
                    },
                },
            }, extType, platform),
        };
    }
    const res = await Promise.all(buildResultConfig.map((info) => getBuildResult(info)));
    // @ts-ignore
    return res.reduce((acc, result) => {
        return ramda_1.default.mergeDeepLeft(acc, result);
    }, {});
}
exports.generateBuildResult = generateBuildResult;
